<?php
/**
 * Process.php
 * 
 * The Process class is meant to simplify the task of processing
 * user submitted forms, redirecting the user to the correct
 * pages if errors are found, or if form is successful, either
 * way. Also handles the logout procedure.
 *
 * LICENSE: This source file is subject to eTech Data Services license available at
 * http://www.etechdataservices.com/app-license
 * 
 * @author		Ernest Ibeh <e.ibeh@etechdataservices.com>
 * @copyright	2016 eTech Data Services
 * @license		http://www.etechdataservices.com/app-license
 */
// Ensure included file exists
$file = "sys/core/session.php";
if ( file_exists($file) && is_readable($file))
{
	include ($file);
} else
{
	$errMsg = "Error: included S file is missing";
}

class Process
{
	/* Class constructor */
	function Process(){
		global $session;
		/* User submitted login form */
		if(isset($_POST['sublogin'])){
			$this->procLogin();
		}
		/* User submitted registration form */
		else if(isset($_POST['subjoin'])){
			$this->procRegister();
		}
		
		/* User submitted registration form */
		else if(isset($_POST['member_subjoin'])){
			$this->procMemberRegister();
		}
	  
		/* User submitted registration form */
		else if(isset($_POST['master_subjoin'])){
		$this->procMasterRegister();
		}
	  
		/* User submitted registration form */
		else if(isset($_POST['agent_subjoin'])){
			$this->procAgentRegister();
		}
	  
		/* User submitted forgot password form */
		else if(isset($_POST['subforgot'])){
			$this->procForgotPass();
		}
		/* User submitted edit account form */
		else if(isset($_POST['subedit'])){
			$this->procEditAccount();
		}
		/**
		* The only other reason user should be directed here
		* is if he wants to logout, which means user is
		* logged in currently.
		*/
		else if($session->logged_in){
			$this->procLogout();
		}
		/**
		* Should not get here, which means user is viewing this page
		* by mistake and therefore is redirected.
		*/
		else{
		header("Location: index");
		}
	}

	/**
	* procLogin - Processes the user submitted login form, if errors
	* are found, the user is redirected to correct the information,
	* if not, the user is effectively logged in to the system.
	*/
	function procLogin(){
		global $session, $form;
		/* Login attempt */
		// url is now stored as $_POST['location'] (posted from login.php). 
		//If it's blank, let's ignore it. Otherwise, let's do something with it.
		$redirect = NULL;
		if($_POST['location'] != '') {
			$redirect = $_POST['location'];
		}
		$retval = $session->login($_POST['user'], $_POST['pass'], $_POST['location'], isset($_POST['remember']));
      
		/* Login successful */
		if($retval && $redirect){
			 header("Location:". $redirect);
		} else if ($retval){
			header("Location: dashboard");
		}
		/* Login failed */
		else{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer);
		}
	}
   
	/**
	* procLogout - Simply attempts to log the user out of the system
	* given that there is no logout form to process.
	*/
	function procLogout(){
		global $session;
		$retval = $session->logout();
		header("Location: index");
	}
   
	/**
	* procRegister - Processes the user submitted registration form,
	* if errors are found, the user is redirected to correct the
	* information, if not, the user is effectively registered with
	* the system and an email is (optionally) sent to the newly
	* created user.
	*/
	function procRegister(){
		global $session, $form;
		/* Convert username to all lowercase (by option) */
		if(ALL_LOWERCASE){
			$_POST['user'] = strtolower($_POST['user']);
		}
		/* Registration attempt */
		$retval = $session->register($_POST['user'], $_POST['pass'], $_POST['email']);
      
		/* Registration Successful */
		if($retval == 0){
			$_SESSION['reguname'] = $_POST['user'];
			$_SESSION['regsuccess'] = true;
			header("Location: index");
		}
		/* Error found with form */
		else if($retval == 1){
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: index");
		}
		/* Registration attempt failed */
		else if($retval == 2){
			$_SESSION['reguname'] = $_POST['user'];
			$_SESSION['regsuccess'] = false;
			header("Location: index");
		}
	}
   
	function procMasterRegister(){
		global $session, $form;
		/* Convert username to all lowercase (by option) */
		if(ALL_LOWERCASE){
			$_POST['user'] = strtolower($_POST['user']);
		}
		/* Registration attempt */
		$retval = $session->SessionMasterRegister($_POST['user'], $_POST['pass'], $_POST['email']);
      
		/* Registration Successful */
		if($retval == 0){
			$_SESSION['reguname'] = $_POST['user'];
			$_SESSION['regsuccess'] = true;
			header("Location: ".$session->referrer.'?'.$session->username);
		}
		/* Error found with form */
		else if($retval == 1){
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer.'?'.$session->username);
		}
		/* Registration attempt failed */
		else if($retval == 2){
			$_SESSION['reguname'] = $_POST['user'];
			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer.'?'.$session->username);
		}
	}

	function procMemberRegister(){
		global $session, $form;
		/* Convert username to all lowercase (by option) */
		if(ALL_LOWERCASE){
			$_POST['user'] = strtolower($_POST['user']);
		}
		/* Registration attempt */
		$retval = $session->SessionMemberRegister($_POST['user'], $_POST['pass'], $_POST['email']);
      
		/* Registration Successful */
		if($retval == 0){
			$_SESSION['reguname'] = $_POST['user'];
			$_SESSION['regsuccess'] = true;
			header("Location: ".$session->referrer.'?'.$session->username);
		}
		/* Error found with form */
		else if($retval == 1){
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer.'?'.$session->username);
		}
		/* Registration attempt failed */
		else if($retval == 2){
			$_SESSION['reguname'] = $_POST['user'];
			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer.'?'.$session->username);
		}
	}

	function procAgentRegister(){
		global $session, $form;
		/* Convert username to all lowercase (by option) */
		if(ALL_LOWERCASE){
			$_POST['user'] = strtolower($_POST['user']);
		}
		/* Registration attempt */
		$retval = $session->SessionAgentRegister($_POST['user'], $_POST['pass'], $_POST['email']);
      
		/* Registration Successful */
		if($retval == 0){
			$_SESSION['reguname'] = $_POST['user'];
			$_SESSION['regsuccess'] = true;
			header("Location: ".$session->referrer.'?'.$session->username);
		}
		/* Error found with form */
		else if($retval == 1){
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: ".$session->referrer.'?'.$session->username);
		}
		/* Registration attempt failed */
		else if($retval == 2){
			$_SESSION['reguname'] = $_POST['user'];
			$_SESSION['regsuccess'] = false;
			header("Location: ".$session->referrer.'?'.$session->username);
		}
	}
	/**
	* procForgotPass - Validates the given username then if
	* everything is fine, a new password is generated and
	* emailed to the address the user gave on sign up.
	*/
	function procForgotPass(){
		global $database, $session, $mailer, $form;
		/* Username error checking */
		$subuser = $_POST['user'];
		$field = "user";  //Use field name for username
		if(!$subuser || strlen($subuser = trim($subuser)) == 0){
			$form->setError($field, "* Username not entered<br>");
		}
		else{
			/* Make sure username is in database */
			$subuser = stripslashes($subuser);
			if(strlen($subuser) < 5 || strlen($subuser) > 30 ||
				!preg_match("/^([0-9a-z])+$/", $subuser) ||
				(!$database->usernameTaken($subuser))){
				$form->setError($field, "* Username does not exist<br>");
			}
		}
      
		/* Errors exist, have user correct them */
		if($form->num_errors > 0){
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
		}
		/* Generate new password and email it to user */
		else{
			/* Generate new password */
			$newpass = $session->generateRandStr(8);
         
			/* Get email of user */
			$usrinf = $database->getUserInfo($subuser);
			$email  = $usrinf['email'];
         
			/* Attempt to send the email with new password */
			if($mailer->sendNewPass($subuser,$email,$newpass)){
				/* Email sent, update database */
				$database->updateUserField($subuser, "password", md5($newpass));
				$_SESSION['forgotpass'] = true;
			}
			/* Email failure, do not change password */
			else{
				$_SESSION['forgotpass'] = false;
			}
		}
      
		header("Location: ".$session->referrer);
	}
   
	/**
	* procEditAccount - Attempts to edit the user's account
	* information, including the password, which must be verified
	* before a change is made.
	*/
	function procEditAccount(){
		global $session, $form;
		/* Account edit attempt */
		$retval = $session->editAccount($_POST['curpass'], $_POST['newpass'], $_POST['email']);

		/* Account edit successful */
		if($retval){
			$_SESSION['useredit'] = true;
			header("Location: useredit");
		}
		/* Error found with form */
		else{
			$_SESSION['value_array'] = $_POST;
			$_SESSION['error_array'] = $form->getErrorArray();
			header("Location: useredit");
		}
	}
};

/* Initialize process */
$process = new Process;

?>
